# coding: utf-8
import sys
from copy import deepcopy

import requests

if sys.version_info.major >= 3:
    from shlex import quote
else:
    from pipes import quote

HEADER_BLOCKLIST = {
    "Content-Length",
}


def parse(
    request_or_response, compressed=False, verify=True, return_it=False, print_it=True
):
    """
    Args:
        request_or_response: requests.models.Request|requests.models.Response
        return_it: False=return None. True=return the string
        print_it: False=Do Nothing. True=Print parsed string to stdout.

    Print:
        curl command
    """
    if isinstance(request_or_response, requests.models.Response):
        request = deepcopy(request_or_response.request)
        connection_pool = request_or_response.connection.get_connection(request.url)
        http_scheme = connection_pool.scheme
        if http_scheme not in ("http", "https"):
            http_scheme = "http"
        host = connection_pool.host
        is_ipv6 = ":" in host
        if is_ipv6:
            host = "[{}]".format(host)
        request.url = "{scheme}://{host}:{port}{path_url}".format(
            scheme=http_scheme,
            host=host,
            port=connection_pool.port,
            path_url=request.path_url,
        )
    elif isinstance(
        request_or_response, (requests.models.Request, requests.models.PreparedRequest)
    ):
        request = deepcopy(request_or_response)
    else:
        raise Exception(
            "`parse` needs a request or response, not {}".format(
                type(request_or_response)
            )
        )

    curl_string = _parse_request(request=request, compressed=compressed, verify=verify)
    if print_it:
        print(curl_string)
    if return_it:
        return curl_string


def _parse_request(request, compressed=False, verify=True):
    parts = [("curl", None), ("-X", request.method)]
    for k, v in sorted(request.headers.items()):
        if k in HEADER_BLOCKLIST:
            continue
        parts += [("-H", "{0}: {1}".format(k, v))]
    if request.body:
        body = request.body
        if isinstance(body, bytes):
            body = body.decode("utf-8")
        parts += [("-d", body)]
    if compressed:
        parts += [("--compressed", None)]
    if not verify:
        parts += [("--insecure", None)]
    parts += [(None, request.url)]
    flat_parts = []
    for k, v in parts:
        if k:
            flat_parts.append(quote(k))
        if v:
            flat_parts.append(quote(v))
    return " ".join(flat_parts)



# coding: utf-8
from requests_to_curl import parse
import re
import unittest

import requests


class Test(unittest.TestCase):
    def test_empty_data(self):
        r = requests.post(
            "http://google.ru",
            headers={"user-agent": "mytest"},
        )
        self.assertEqual(
            parse(r.request, return_it=True, print_it=False),
            (
                "curl -X POST "
                "-H 'Accept: */*' "
                "-H 'Accept-Encoding: gzip, deflate' "
                "-H 'Connection: keep-alive' "
                "-H 'user-agent: mytest' "
                "http://google.ru/"
            )
        )
    
    def test_response(self):
        r = requests.post(
            "http://google.ru",
            headers={"user-agent": "mytest"},
        )
        self.assertEqual(
            parse(r, return_it=True, print_it=False),
            (
                "curl -X POST "
                "-H 'Accept: */*' "
                "-H 'Accept-Encoding: gzip, deflate' "
                "-H 'Connection: keep-alive' "
                "-H 'user-agent: mytest' "
                "http://google.ru:80/"
            )
        )


    def test_ok(self):
        r = requests.get(
            "http://google.ru",
            data={"a": "b"},
            cookies={"foo": "bar"},
            headers={"user-agent": "mytest"},
        )
        self.assertEqual(
            parse(r.request, return_it=True, print_it=False),
            (
                "curl -X GET "
                "-H 'Accept: */*' "
                "-H 'Accept-Encoding: gzip, deflate' "
                "-H 'Connection: keep-alive' "
                "-H 'Content-Type: application/x-www-form-urlencoded' "
                "-H 'Cookie: foo=bar' "
                "-H 'user-agent: mytest' "
                "-d a=b http://google.ru/"
            )
        )


    def test_prepare_request(self):
        request = requests.Request(
            'GET', "http://google.ru",
            headers={"user-agent": "UA"},
        )

        self.assertEqual(
            parse(request.prepare(), return_it=True, print_it=False),
            (
                "curl -X GET "
                "-H 'user-agent: UA' "
                "http://google.ru/"
            )
        )


    def test_compressed(self):
        request = requests.Request(
            'GET', "http://google.ru",
            headers={"user-agent": "UA"},
        )
        self.assertEqual(
            parse(request.prepare(), compressed=True, return_it=True, print_it=False),
            "curl -X GET -H 'user-agent: UA' --compressed http://google.ru/",
        )


    def test_verify(self):
        request = requests.Request(
            'GET', "http://google.ru",
            headers={"user-agent": "UA"},
        )
        self.assertEqual(
            parse(request.prepare(), verify=False, return_it=True, print_it=False),
            "curl -X GET -H 'user-agent: UA' --insecure http://google.ru/",
        )


    def test_post_json(self):
        data = {'foo': 'bar'}
        url = 'https://httpbin.org/post'

        r = requests.Request('POST', url, json=data)
        curlified = parse(r.prepare(), return_it=True, print_it=False)

        self.assertEqual(curlified, (
            "curl -X POST "
            "-H 'Content-Type: application/json' "
            "-d '{\"foo\": \"bar\"}' https://httpbin.org/post"
        ))


    def test_post_csv_file(self):
        r = requests.Request(
            method='POST',
            url='https://httpbin.org/post',
            files={'file': open('data.csv', 'r')},
            headers={'User-agent': 'UA'}
        )

        curlified = parse(r.prepare(), return_it=True, print_it=False)
        boundary = re.search(r'boundary=(\w+)', curlified).group(1)
        expected = (
            'curl -X POST'
            f' -H \'Content-Type: multipart/form-data; boundary={boundary}\''
            ' -H \'User-agent: UA\''
            f' -d \'--{boundary}\r\nContent-Disposition: form-data; name="file"; filename="data.csv"\r\n\r\n'
            '"Id";"Title";"Content"\n'
            '1;"Simple Test";"Ici un test d\'"\'"\'échappement de simple quote"\n'
            '2;"UTF-8 Test";"ăѣ𝔠ծềſģȟᎥ𝒋ǩľḿꞑȯ𝘱𝑞𝗋𝘴ȶ𝞄𝜈ψ𝒙𝘆𝚣1234567890!@#$%^&*()-_=+;:\'"\'"\'",[]{}<.>/?~𝘈Ḇ𝖢𝕯٤ḞԍНǏ𝙅ƘԸⲘ𝙉০Ρ𝗤Ɍ𝓢ȚЦ𝒱Ѡ𝓧ƳȤѧᖯć𝗱ễ𝑓𝙜Ⴙ𝞲𝑗𝒌ļṃŉо𝞎𝒒ᵲꜱ𝙩ừ𝗏ŵ𝒙𝒚ź"'
            f'\r\n--{boundary}--\r\n\''
            ' https://httpbin.org/post'
        )
        self.assertEqual(curlified, expected)


unittest.main()
